﻿-- Work Order Tooltips
-- by Jadya - EU-Well of Eternity

local addonName, addonTable = ...
local L = addonTable.L

local ng = NyxGUI("1.0")

local max, sfind, tinsert, tremove = math.max, string.find, table.insert, table.remove

local cvar_queue = {}
local addon_enabled, garrison
local classIndex = select(3, UnitClass("player"))

local tt_w1, tt_w2 = 220, 175
local tt_h1, tt_h2 = 120, 40

local db = addonTable.db

local hookedframes = {}
local pool = {}
local function getframe()
 local f = pool[1]
 if f then
  tremove(pool, 1) 
 else
  f = ng:New(addonName, "Frame")
 end
 return f
end

function addonTable:updateTooltipsScale(v)
 for _, tt in pairs(hookedframes) do
  tt:SetScale(v)
 end
end
--[[
function addonTable:updateTooltipsAlpha(v)
 for _, tt in pairs(hookedframes) do
  if tt.name ~= UnitName("target") then
   tt:SetAlpha(v)
  end
 end
end
]]
local lastupdate = 0
local alphaupdate = 0
local function updateTooltip(tt, elapsed, forced)
 
 alphaupdate = alphaupdate + elapsed
 if alphaupdate > 0.06 then
  alphaupdate = 0
  local name = UnitName("target")
  local targetAlpha
  if name == tt.name then
   targetAlpha = 1
  else
   targetAlpha = WOTSaves.alpha
  end

  if tt.alpha < targetAlpha then
   tt.alpha = (tt.alpha < targetAlpha - 0.04) and (tt.alpha + 0.04) or targetAlpha
   tt:SetAlpha(tt.alpha)
  elseif tt.alpha > targetAlpha then
   tt.alpha = (tt.alpha > targetAlpha + 0.04) and (tt.alpha - 0.04) or targetAlpha
   tt:SetAlpha(tt.alpha)
  end
 end

 lastupdate = lastupdate + elapsed
 if lastupdate > 1 or forced then lastupdate = 0 else return end

 if tt.id == 0 then
  tt:Clear()
  tt.title:SetText(QUEUE_TIME_UNAVAILABLE)
  return
 end

 local bName, bIcon, _, completedWO, currentWO, _, WOduration, currWOtimeleft, WIname, WIicon
 if tt.t == 1 then
  bName, bIcon, _, completedWO, currentWO, _, WOduration, currWOtimeleft, WIname, WIicon = C_Garrison.GetLandingPageShipmentInfoByContainerID(tt.id)
 elseif tt.t == 2 then
  local talentTrees = C_Garrison.GetTalentTrees(LE_GARRISON_TYPE_7_0, classIndex)
  local completeTalentID = C_Garrison.GetCompleteTalent(LE_GARRISON_TYPE_7_0)
  if talentTrees then
   for treeIndex, tree in ipairs(talentTrees) do
    for talentIndex, talent in ipairs(tree) do
  	 local showTalent = talent.isBeingResearched or talent.id == completeTalentID
  	 if showTalent then
	  bName  = TALENT
 	  WIname = talent.name
	  WIicon = talent.icon
	  currentWO = 1
	  maxWO = 1
	  completedWO = talent.id == completeTalentID and 1 or 0
	  currWOtimeleft = SecondsToTime(talent.researchStartTime + talent.researchDuration - time())
   	 end
    end
   end
  end
 end

 if not bName then
  tt:Clear()
  tt.title:SetText(GARRISON_FOLLOWER_INACTIVE)
  if tt.height == tt_h1 then
   tt:SetHeight(tt_h2)
   tt.height = tt_h2
  end
  if tt.width == tt_w1 then
   tt:SetWidth(tt_w2)
   tt.width = tt_w2
  end
  return
 end
 
 if tt.height == tt_h2 then
  tt:SetHeight(tt_h1)
  tt.height = tt_h1
 end
 if tt.width == tt_w2 then
  tt:SetWidth(tt_w1)
  tt.width = tt_w1
 end
 
 local loot_string = WIname and ("- |T"..WIicon..":12|t "..WIname.." - ") or ""
 local queued_string = currentWO and currentWO - completedWO or 0

 tt.title:SetText("|cffcccc00"..bName.."|r")
 tt.leftfont:SetText(READY..": \n"
                   ..BATTLEFIELD_QUEUE_STATUS..": \n"
                   ..NEXT..": " .."\n\n"
                   ..loot_string)
 tt.rightfont:SetText("|cff00FF00"..(completedWO or 0).."\n"
                    ..queued_string.."\n"
                    ..(currWOtimeleft or NOT_APPLICABLE).."|r")
end

local function buildTooltip(f, t, name, unitid)
 if not WOTSaves.npcs[name] then return end

 local id = WOTSaves.npcs[name].id
 local tt = getframe()

 if not tt.initialized then
  tt:SetSize(tt_w1, tt_h1)
  tt.width = tt_w1
  tt.height = tt_h1
  tt:SetPoint("BOTTOM", f, "TOP")
  tt:SetFrameStrata("LOW")
  tt:SetFrameLevel(1)

  tt.title = ng:New(addonName, "Label", nil, tt, "title")
  tt.title:SetPoint("TOPLEFT", tt, "TOPLEFT", 5, -5)
  tt.title:SetPoint("BOTTOMRIGHT", tt, "TOPRIGHT", -5, -25)
  
  tt.leftfont = ng:New(addonName, "Label", nil, tt)
  tt.leftfont:SetJustifyH("LEFT")
  tt.leftfont:SetPoint("TOPLEFT", tt, "TOPLEFT", 5, -30)
  tt.leftfont:SetPoint("BOTTOMRIGHT", tt, "BOTTOMRIGHT", -5, 5)
  
  tt.rightfont = ng:New(addonName, "Label", nil, tt)
  tt.rightfont:SetJustifyH("RIGHT")
  tt.rightfont:SetPoint("TOPRIGHT",-5,-30)
  
  tt.Clear = function()
   tt.title:SetText("")
   tt.rightfont:SetText("")
   tt.leftfont:SetText("")
  end
  
  tt.initialized = true
 end

 tt.t = t
 tt.id = id
 tt:Clear()
 tt:SetScale(WOTSaves.scale or 1)
 tt:SetParent(UIParent)
 tt.name = name
 tt.unitid = unitid
 tt.alpha = WOTSaves.alpha
 tt:SetAlpha(tt.alpha)
 
 tt:SetScript("OnUpdate", function(self, elapsed) updateTooltip(self, elapsed) end)
 tt:SetPoint("BOTTOM", f, "TOP")
 updateTooltip(tt, 0, true)
 tt:Show()
 return tt
end

local function nameplateRemoved(id)
 if hookedframes[id] then
  hookedframes[id].name = ""
  hookedframes[id]:SetParent(nil)
  hookedframes[id]:Hide()
  pool[#pool + 1] = hookedframes[id]
  hookedframes[id] = nil
 end
end

local function nameplateAdded(id)
 local f = C_NamePlate.GetNamePlateForUnit(id)
 
 local name = UnitName(id)

 if not f or not name then return end
 
 if WOTSaves.npcs[name] then
  hookedframes[id] = buildTooltip(f, WOTSaves.npcs[name].type, name, id)
 end
end

function checkNamePlates()
 for k,v in pairs(C_NamePlate.GetNamePlates()) do
  local id = v.namePlateUnitToken
  local name = UnitName(id)

  if id and not hookedframes[id] and name and WOTSaves.npcs[name] then
   hookedframes[id] = buildTooltip(v, WOTSaves.npcs[name].type, name, id)
  end
 end
end

local function setTalentID()
 local name = UnitName("target")

 if not name then return end
 
 if not WOTSaves.npcs[name] then WOTSaves.npcs[name] = {} end
 local t = WOTSaves.npcs[name]
 t.type = 2
 checkNamePlates()
end

local function setContainerID(containerID)
 local name = UnitName("target")
 
 if not name then return end
 
 if not WOTSaves.npcs[name] then WOTSaves.npcs[name] = {} end
 
 for k,v in pairs(WOTSaves.npcs) do
  if v.id == containerID and k ~= name then
   WOTSaves.npcs[k] = nil
  end
 end

 local t = WOTSaves.npcs[name]
 t.id = containerID
 t.type = 1
 checkNamePlates()
end

local function set_cvar(k, v)
 if InCombatLockdown() then
  cvar_queue[k] = v
 else
  if tonumber(GetCVar(k)) ~= v then
   SetCVar(k, v)
  end
 end
end

function addonTable:nameplate_var_changed(t, v)
 if v == 3 then return end

 local pref = t:sub(1, 3)
 
 if (pref == "in_" and garrison) or (pref == "out" and not garrison) then
  if t == pref.."_friendly" then
   set_cvar("nameplateShowFriends", v == 1 and 1 or 0)
  elseif t == pref.."_showall" then
   set_cvar("nameplateShowAll",  v == 1 and 1 or 0)
  end
 end
end

local eventframe = CreateFrame("Frame")
local function checkGarrison()

 garrison = nil
 if not C_Garrison.IsUsingPartyGarrison() then
  for i = 2,3 do
   if C_Garrison.IsPlayerInGarrison(i) then
    garrison = i
    break
   end
  end
 end

 if garrison and not addon_enabled then
  eventframe:RegisterEvent("SHIPMENT_CRAFTER_CLOSED")
  eventframe:RegisterEvent("CHAT_MSG_LOOT")
  eventframe:RegisterEvent("CHAT_MSG_CURRENCY")
  eventframe:RegisterEvent("NAME_PLATE_UNIT_ADDED")
  eventframe:RegisterEvent("NAME_PLATE_UNIT_REMOVED")
  eventframe:RegisterEvent("UNIT_TARGET")
  eventframe:RegisterEvent("UPDATE_MOUSEOVER_UNIT")
  eventframe:RegisterEvent("SHIPMENT_CRAFTER_OPENED")
  eventframe:RegisterEvent("GARRISON_TALENT_NPC_OPENED")
  addon_enabled = true
  addonTable:nameplate_var_changed("in__friendly", WOTSaves.in__friendly)
  addonTable:nameplate_var_changed("in__showall", WOTSaves.in__showall)
 elseif addon_enabled and not garrison then
  eventframe:UnregisterEvent("SHIPMENT_CRAFTER_CLOSED")
  eventframe:UnregisterEvent("CHAT_MSG_LOOT")
  eventframe:UnregisterEvent("CHAT_MSG_CURRENCY")
  eventframe:UnregisterEvent("NAME_PLATE_UNIT_ADDED")
  eventframe:UnregisterEvent("NAME_PLATE_UNIT_REMOVED")
  eventframe:UnregisterEvent("UNIT_TARGET")
  eventframe:UnregisterEvent("UPDATE_MOUSEOVER_UNIT")
  eventframe:UnregisterEvent("SHIPMENT_CRAFTER_OPENED")
  eventframe:UnregisterEvent("GARRISON_TALENT_NPC_OPENED")
  addonTable:nameplate_var_changed("out_friendly", WOTSaves.out_friendly)
  addonTable:nameplate_var_changed("out_showall", WOTSaves.out_showall)
  addon_enabled = false
 end
end

eventframe:RegisterEvent("VARIABLES_LOADED")
eventframe:RegisterEvent("GARRISON_UPDATE")
eventframe:RegisterEvent("PLAYER_REGEN_ENABLED")
local lastT = 0
local function eventhandler(self, event, ...)
 if event == "VARIABLES_LOADED" then
  if not WOTSaves then WOTSaves = {} end
  if not WOTSaves.npcs then WOTSaves.npcs = {} end
  
  WOTSaves.showplates_ingarr = nil
  WOTSaves.changeplates = nil
  
  if WOTSaves.in__showall  == nil then WOTSaves.in__showall  = 3 end
  if WOTSaves.out_showall  == nil then WOTSaves.out_showall  = 3 end
  if WOTSaves.in__friendly == nil then WOTSaves.in__friendly = 1 end
  if WOTSaves.out_friendly == nil then WOTSaves.out_friendly = 2 end
  if WOTSaves.alpha == nil        then WOTSaves.alpha = 1 end
  if WOTSaves.scale == nil        then WOTSaves.scale = 1 end

   local theme = {
               -- textures and colors
			   l0_texture     = "Interface\\Buttons\\GreyscaleRamp64",
               l0_color       = "111111e6",
			   l0_border      = "7f7f19e6",
			  -- fonts
			   f_label_h      = 14,
			  }
  
  ng:Initialize(addonName, WOTSaves, nil, theme)
  ng:LoadTheme(addonName, "title", { f_label_name = "Fonts\\MORPHEUS.ttf", f_label_h = 18 } )
  ng:SetThemeData(addonName, "title", { f_label_name = "Fonts\\MORPHEUS.ttf", f_label_h = 18 } )

  C_Timer.After(1, function()
    checkGarrison()
	if addon_enabled then
	 --C_Garrison.RequestLandingPageShipmentInfo()
	 checkNamePlates()
	end
   end)
 elseif addon_enabled and (event == "SHIPMENT_CRAFTER_CLOSED" or event == "CHAT_MSG_LOOT" or event == "CHAT_MSG_CURRENCY") and lastT < time() - 1 then
  lastT = time()
  --C_Garrison.RequestLandingPageShipmentInfo()
 elseif event == "GARRISON_UPDATE" then
  checkGarrison()
 elseif event == "NAME_PLATE_UNIT_ADDED" then
  nameplateAdded(...)
 elseif event == "NAME_PLATE_UNIT_REMOVED" then
  nameplateRemoved(...)
 elseif event == "SHIPMENT_CRAFTER_OPENED" then
  setContainerID(...)
 elseif event == "GARRISON_TALENT_NPC_OPENED" then
  setTalentID()
 elseif event == "PLAYER_REGEN_ENABLED" then
  for k,v in pairs(cvar_queue) do
   set_cvar(k, v)
  end
  wipe(cvar_queue)
 end
end
eventframe:SetScript("OnEvent", eventhandler)